@@ -1,7 +1,7 @@
 /*************************************************************************
 * libsqlxx                                                               *
 * ---------------------------------------------------------------------- *
-* Copyright (C) 2000,2001  Klaus Reimer, k@ailis.de                      *
+* Copyright (C) 2000-2002  Klaus Reimer, k@ailis.de                      *
 *                                                                        *
 * This library is free software; you can redistribute it and/or          *
 * modify it under the terms of the GNU Lesser General Public             *
@@ -20,8 +20,8 @@
 *************************************************************************/
 
 #include "sqlxx.h"
-#include <algo.h>
-#include <iostream.h>
+#include <iostream>
+#include <algorithm>
 #ifdef USE_MYSQL
 #include <mysql.h>
 #endif
@@ -40,14 +40,18 @@ namespace sqlxx {
 
 #ifdef DEBUG
 int ICDI=0;
-#define TRACEINPARAM(c,m) for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "-> sqlxx: " << this << " = " << c << "(" << m << ")" << endl; ICDI++;
-#define TRACEIN(c) for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "-> sqlxx: " << this << " = " << c << "()" << endl; ICDI++;
-#define TRACEOUTRET(c,r) ICDI--; for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "<- sqlxx: " << this << " = " << c << "=" << r << endl;
-#define TRACEOUT(c) ICDI--; for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "<- sqlxx: " << this << " = " << c << endl;
+#define TRACEINPARAM(c,m) for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "-> sqlxx: " << this << " = " << c << "(" << m << ")" << endl; ICDI++; fflush(stderr);
+#define FTRACEINPARAM(c,m) for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "-> sqlxx: " << c << "(" << m << ")" << endl; ICDI++; fflush(stderr);
+#define TRACEIN(c) for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "-> sqlxx: " << this << " = " << c << "()" << endl; ICDI++; fflush(stderr);
+#define TRACEOUTRET(c,r) ICDI--; for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "<- sqlxx: " << this << " = " << c << "=" << r << endl; fflush(stderr);
+#define FTRACEOUTRET(c,r) ICDI--; for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "<- sqlxx: " << " = " << c << "=" << r << endl; fflush(stderr);
+#define TRACEOUT(c) ICDI--; for(int IDI=0;IDI<ICDI;IDI++) cerr << "  "; cerr << "<- sqlxx: " << this << " = " << c << endl; fflush(stderr);
 #else
 #define TRACEINPARAM(c,m)
+#define FTRACEINPARAM(c,m)
 #define TRACEIN(c)
 #define TRACEOUTRET(c,r)
+#define FTRACEOUTRET(c,r)
 #define TRACEOUT(c)
 #endif
 
@@ -68,7 +72,7 @@ int ICDI=0;
     CSQL::openQuery returns an instance of this class.
 */
 
-CSQLResult::CSQLResult(CSQL *Database, const int iBufferSize=1024) {
+CSQLResult::CSQLResult(CSQL *Database, const int iBufferSize) {
   TRACEINPARAM("CSQLResult::CSQLResult",format("%p,%i",Database,iBufferSize))
   this->Database=Database;
   this->iBufferSize=iBufferSize;
@@ -512,6 +516,8 @@ bool CSQLResult::fetch(void) {
 
 string CSQLResult::get(const unsigned int iIndex) {
   TRACEINPARAM("CSQLResult::get",iIndex)
+  if (!vResult.size())
+    throw sqlxx_error("CSQLResult::get: No data available");
   if (iIndex>=iNumCols)
     throw sqlxx_error("CSQLResult::get: Invalid index: "+intstr(iIndex));
   TRACEOUTRET("CSQLResult::get",vResult[iIndex].sValue)
@@ -527,6 +533,8 @@ string CSQLResult::get(const string sField) {
   TRACEINPARAM("CSQLResult::get",sField)
   unsigned int iIndex;
   
+  if (!vResult.size())
+    throw sqlxx_error("CSQLResult::get: No data available");
   iIndex=0;
   while ((iIndex<iNumCols) && (vFields[iIndex]!=sField)) iIndex++;
   if (iIndex>=iNumCols)
@@ -544,6 +552,8 @@ long CSQLResult::getInt(const unsigned int iIndex) {
   int iValue;
   
   TRACEINPARAM("CSQLResult::getInt",iIndex)
+  if (!vResult.size())
+    throw sqlxx_error("CSQLResult::getInt: No data available");
   if (iIndex>=iNumCols)
     throw sqlxx_error("CSQLResult::getInt: Invalid index: "+intstr(iIndex));
   iValue=atoi(vResult[iIndex].sValue.c_str());
@@ -561,6 +571,8 @@ long CSQLResult::getInt(const string sField) {
   unsigned int iIndex;
   
   TRACEINPARAM("CSQLResult::getInt",sField)
+  if (!vResult.size())
+    throw sqlxx_error("CSQLResult::getInt: No data available");
   iIndex=0;
   while ((iIndex<iNumCols) && (vFields[iIndex]!=sField)) iIndex++;
   if (iIndex>=iNumCols)
@@ -579,6 +591,8 @@ float CSQLResult::getFloat(const unsigned int iIndex) {
   float fValue;
   
   TRACEINPARAM("CSQLResult::getFloat",iIndex)
+  if (!vResult.size())
+    throw sqlxx_error("CSQLResult::getFloat: No data available");
   if (iIndex>=iNumCols)
     throw sqlxx_error("CSQLResult::getFloat: Invalid index: "+intstr(iIndex));
   fValue=atof(vResult[iIndex].sValue.c_str());
@@ -596,6 +610,8 @@ float CSQLResult::getFloat(const string sField) {
   unsigned int iIndex;
   
   TRACEINPARAM("CSQLResult::getFloat",sField)
+  if (!vResult.size())
+    throw sqlxx_error("CSQLResult::getFloat: No data available");
   iIndex=0;
   while ((iIndex<iNumCols) && (vFields[iIndex]!=sField)) iIndex++;
   if (iIndex>=iNumCols)
@@ -613,6 +629,8 @@ float CSQLResult::getFloat(const string sField) {
 
 bool CSQLResult::isNull(const unsigned int iIndex) {
   TRACEINPARAM("CSQLResult::isNull",iIndex)
+  if (!vResult.size())
+    throw sqlxx_error("CSQLResult::isNull: No data available");
   if (iIndex>=iNumCols)
     throw sqlxx_error("CSQLResult::isNull: Invalid index: "+intstr(iIndex));
   TRACEOUTRET("CSQLResult::isNull",vResult[iIndex].bNull)
@@ -629,6 +647,8 @@ bool CSQLResult::isNull(const string sField) {
   unsigned int iIndex;
   
   TRACEINPARAM("CSQLResult::isNull",sField)
+  if (!vResult.size())
+    throw sqlxx_error("CSQLResult::isNull: No data available");
   iIndex=0;
   while ((iIndex<iNumCols) && (vFields[iIndex]!=sField)) iIndex++;
   if (iIndex>=iNumCols)
@@ -694,7 +714,7 @@ CSQL::~CSQL(void) {
     already contains the complete error message. 
 */
 
-string CSQL::getError(void *Result=NULL) { 
+string CSQL::getError(void *Result) { 
   #ifdef USE_ODBC
   char cSQLState[15];
   SDWORD iNativeError;
@@ -1216,7 +1236,7 @@ bool CSQL::isConnected(void) {
     a CSQLResult object.
 */
 
-CSQLResult *CSQL::openQuery(const string &sQuery, const int iBufferSize=1024) {
+CSQLResult *CSQL::openQuery(const string &sQuery, const int iBufferSize) {
   CSQLResult *DBResult;
   
   TRACEINPARAM("CSQL::openQuery",format("%s,%i",sQuery.c_str(),iBufferSize))
@@ -1300,4 +1320,23 @@ int CSQL::countQueries(void) {
   return iResult;
 }
 
+
+/** Quotes a string.
+    Quotes the string sSource and returns this new quoted string.  This
+    method uses the addSlashes function of the strutilsxx library so there
+    are another two optional parameters: bEscape8Bit, sEscape. They are
+    relayed to the addSlashes function and are explained in the strutilsxx
+    documentation.
+*/
+
+string quote(const string &sSource, const bool bEscape8Bit, 
+    const string &sEscape) {
+  string sResult;
+  
+  FTRACEINPARAM("quote",format("%s,%i,%s",sSource.c_str(),bEscape8Bit,sEscape.c_str()))
+  sResult = "'" + addSlashes(sSource, bEscape8Bit, sEscape) + "'";
+  FTRACEOUTRET("CSQL::quote",sResult)
+  return sResult;
+}
+
 }
